// Copyright (C) 2011-2013 by Maximilian Hönig
// This file is part of "DunGen - the Dungeongenerator".
// For conditions of distribution and use, see licence.txt provided together with DunGen.

#include "interface/DunGen.h"
#include "DungeonGenerator.h"
#include "LSystem.h"
#include "MeshCave.h"
#include "VoxelCave.h"

DunGen::CDunGen::CDunGen( irr::IrrlichtDevice* irrDevice)
{
	DungeonGenerator = new CDungeonGenerator(irrDevice);
}

DunGen::CDunGen::~CDunGen()
{
	delete DungeonGenerator;
}

unsigned int DunGen::CDunGen::CreateLSystemDerivation(unsigned int desiredIteration)
{
	return DungeonGenerator->CreateLSystemDerivation(desiredIteration);
}

void DunGen::CDunGen::CreateVoxelCave()
{
	DungeonGenerator->CreateVoxelCave();
}

void DunGen::CDunGen::ErodeVoxelCave(double erosionLikelihood)
{
	DungeonGenerator->ErodeVoxelCave(erosionLikelihood);
}

void DunGen::CDunGen::RemoveHoveringVoxelFragments()
{
	DungeonGenerator->RemoveHoveringVoxelFragments();
}

void DunGen::CDunGen::CreateMeshCave()
{
	DungeonGenerator->CreateMeshCave();
}

bool DunGen::CDunGen::CreateRoom(unsigned int roompatternIndex, const irr::core::vector3d<double>& position,
	const irr::core::vector3d<double>& rotation, const irr::core::vector3d<double>& scaleFactor)
{
	return DungeonGenerator->CreateRoom(roompatternIndex, position, rotation, scaleFactor);
}

bool DunGen::CDunGen::CreateCorridorRoomRoom(
	unsigned int room0, unsigned int dockingSite0, double distance0, double strenght0,
	unsigned int room1, unsigned int dockingSite1, double distance1, double strenght1,
	bool& sightBlocking)
{
	return DungeonGenerator->CreateCorridorRoomRoom(
		room0, dockingSite0, distance0, strenght0, room1, dockingSite1, distance1, strenght1, sightBlocking);
}

bool DunGen::CDunGen::CreateCorridorRoomCave(
	unsigned int room0, unsigned int dockingSite0, double distance0, double strenght0,
	const irr::core::vector3d<unsigned int>& minVox1, const irr::core::vector3d<unsigned int>& maxVox1,
	EDirection::Enum direction1, double distance1, double strenght1,
	bool& sightBlocking)
{
	return DungeonGenerator->CreateCorridorRoomCave(
		room0, dockingSite0, distance0, strenght0, minVox1, maxVox1, direction1, distance1, strenght1, sightBlocking);
}

bool DunGen::CDunGen::CreateCorridorCaveCave(
	const irr::core::vector3d<unsigned int>& minVox0, const irr::core::vector3d<unsigned int>& maxVox0,
	EDirection::Enum direction0, double distance0, double strenght0,
	const irr::core::vector3d<unsigned int>& minVox1, const irr::core::vector3d<unsigned int>& maxVox1,
	EDirection::Enum direction1, double distance1, double strenght1,
	bool& sightBlocking)
{
	return DungeonGenerator->CreateCorridorCaveCave(
		minVox0, maxVox0, direction0, distance0, strenght0, minVox1, maxVox1, direction1, distance1, strenght1, sightBlocking);
}

void DunGen::CDunGen::AddDungeon(irr::scene::ISceneNode* parentNode, irr::scene::ISceneManager* sceneManager)
{
	DungeonGenerator->AddDungeon(parentNode, sceneManager);
}

unsigned char (&DunGen::CDunGen::GetVoxelSpace())[SVoxelSpace::DimX][SVoxelSpace::DimY][SVoxelSpace::DimZ]
{
	return DungeonGenerator->GetVoxelSpace();
};

void DunGen::CDunGen::RandomGeneratorSetParameters(unsigned int seed, unsigned int a, unsigned int c, unsigned int m)
{
	DungeonGenerator->RandomGeneratorSetParameters(seed, a, c, m);
}

void DunGen::CDunGen::LSystemAddRule(char symbol, const std::string& substitution)
{
	DungeonGenerator->GetLSystem().AddRule(symbol, substitution);
}

void DunGen::CDunGen::LSystemDeleteRules()
{
	DungeonGenerator->GetLSystem().DeleteRules();
}

void DunGen::CDunGen::LSystemSetStart(const std::string& start)
{
	DungeonGenerator->GetLSystem().SetStart(start);
}

void DunGen::CDunGen::LSystemSetParameter(ELSystemParameter::Enum parameter, double value)
{
	DungeonGenerator->GetLSystem().SetParameter(parameter, value);
}

void DunGen::CDunGen::VoxelCaveSetParameters(unsigned int border, unsigned int mindrawradius)
{
	DungeonGenerator->GetVoxelCave().SetBorder(border);
	DungeonGenerator->GetVoxelCave().SetMinDrawRadius(mindrawradius);
}

unsigned int DunGen::CDunGen::VoxelCaveEstimateMeshComplexity() const
{
	return DungeonGenerator->GetVoxelCave().EstimateMeshComplexity();
}

void DunGen::CDunGen::MeshCaveSetWarpParameters(bool warpEnabled, bool smoothEnabled, unsigned int warpRandomSeed, double warpStrength)
{
	DungeonGenerator->GetMeshCave().SetWarpOption(warpEnabled);
	DungeonGenerator->GetMeshCave().SetSmoothOption(smoothEnabled);
	DungeonGenerator->GetMeshCave().SetWarpRandomSeed(warpRandomSeed);
	DungeonGenerator->GetMeshCave().SetWarpStrength(warpStrength);
}

void DunGen::CDunGen::MeshCaveSetNormalWeightMethod(ENormalWeightMethod::Enum value)
{
	DungeonGenerator->GetMeshCave().SetNormalWeightMethod(value);
}

void DunGen::CDunGen::CorridorSetDistances(double distance, double textureDistance)
{
	DungeonGenerator->CorridorSetDistances(distance,textureDistance);
}

void DunGen::CDunGen::CorrdidorAddPoint(double x, double y, double textureX)
{
	DungeonGenerator->CorrdidorAddPoint(x, y, textureX);
}

void DunGen::CDunGen::CorrdidorRemovePoints()
{
	DungeonGenerator->CorrdidorRemovePoints();
}

void DunGen::CDunGen::CorrdidorAddDetailObject(const SDetailobjectParameters& parameters)
{
	DungeonGenerator->CorrdidorAddDetailObject(parameters);
}

void DunGen::CDunGen::CorrdidorRemoveDetailobjects()
{
	DungeonGenerator->CorrdidorRemoveDetailobjects();
}

void DunGen::CDunGen::RoomPatternLoad(const irr::io::path filename)
{
	DungeonGenerator->RoomPatternLoad(filename);
}

void DunGen::CDunGen::MaterialSetCorridor(const irr::io::path& textureFilename, bool backFaceCulling, bool antiAliasing)
{
	DungeonGenerator->MaterialSetCorridor(textureFilename, backFaceCulling, antiAliasing);
}

void DunGen::CDunGen::MaterialSetCaveSingleColor(const irr::video::SColorf& color, bool backFaceCulling, bool antiAliasing)
{
	DungeonGenerator->MaterialSetCaveSingleColor(color, backFaceCulling, antiAliasing);
}

void DunGen::CDunGen::MaterialSetCaveMultiColor(bool backFaceCulling, bool antiAliasing)
{
	DungeonGenerator->MaterialSetCaveMultiColor(backFaceCulling, antiAliasing);
}

void DunGen::CDunGen::SetPrintToConsole(bool value)
{
	DungeonGenerator->SetPrintToConsole(value);
}